"""
$URL: svn+ssh://svn.mems-exchange.org/repos/trunk/durus/__init__.py $
$Id: __init__.py 28563 2006-08-02 14:23:42Z dbinger $
Copyright (c) Corporation for National Research Initiatives 2006. All Rights Reserved.
"""
__version__ = '3.5'
